/* outletctl - see szInfo below for copyright and licence details */

const char szInfo[] = "\
Remote Outlet Control Client\n\
Version 1.0\n\
\n\
A simple client to send commands to the Remote Outlet Control Server.\n\
See http://www.sconemad.com for more information.\n\
\n\
Copyright (c) 2010 Andrew Wedgbury <wedge@sconemad.com>\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2, or (at your option)\n\
any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program (see the file COPYING); if not, write to the\n\
Free Software Foundation, Inc.,\n\
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA\n\
";

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include <string>
#include <iostream>
#include <getopt.h>
#include <errno.h>

short port = 7900;

#define HIBYTE(a) (unsigned char)((a & 0x00FF << 8))
#define LOBYTE(a) (unsigned char)(a & 0xFF)

//=============================================================================
void exit_error(const std::string& msg)
{
  std::cerr << "outletctl: " << msg
            << " (errno=" << errno << ")"
            << "\n";
  exit(1);
}

//=============================================================================
int main(int argc,char* argv[])
{
  // Look for version/help options initially as a special case
  if (argc < 4) {
    std::string arg = (argc > 1) ? argv[1] : "";
    if (arg == "-v" || arg == "--version") {
      std::cout << szInfo;
      return 0;
    }

    std::cout
      << "Usage: outletctl <addr> id button state\n"
      << "where:\n"
      << " addr = Address of server (optional; defaults to loopback)\n"
      << " id = identifier (0...65535)\n"
      << " button = A,B,C,D or ALL\n"
      << " state = ON or OFF\n"
      << "\n";
    return 0;
  }

  int argi = 0;
  // Server address
  char addr[64];
  if (argc == 5) {
    strcpy(addr,argv[++argi]);
  } else {
    strcpy(addr,"127.0.0.1");
  }

  // Identifier

  char* str_id = argv[++argi];
  unsigned short id = atoi(str_id);

  // Button
  char* str_btn = argv[++argi];
  unsigned char cmd = 0;

  if (0 == strcmp(str_btn,"a")) cmd = 0xE0;
  else if (0 == strcmp(str_btn,"b")) cmd = 0xF0;
  else if (0 == strcmp(str_btn,"c")) cmd = 0x00;
  else if (0 == strcmp(str_btn,"d")) cmd = 0x10;
  else if (0 == strcmp(str_btn,"all")) cmd = 0x20;
  else {
    char msg[64];
    sprintf(msg,"Invalid button argument '%s'\n",str_btn);
    exit_error(msg);
  }

  // State
  char* str_state = argv[++argi];
  if (0 == strcmp(str_state,"on")) cmd |= 0x04;
  else if (0 == strcmp(str_state,"off")) cmd |= 0x03;
  else {
    char msg[64];
    sprintf(msg,"Invalid state argument '%s'\n",str_state);
    exit_error(msg);
  }

  //----
  int sock = socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP);
  if (sock < 0) {
    exit_error("socket() failed");
  }

  sockaddr_in sa;
  memset(&sa,0,sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons(port);
  if (isalpha(addr[0])) {
    hostent* host = gethostbyname(addr);
    if (host == 0) {
      exit_error("Unable to resolve server address");
    }
    memcpy(&sa.sin_addr.s_addr,*host->h_addr_list,host->h_length);
  } else {
    if (inet_pton(AF_INET,addr,&sa.sin_addr.s_addr) <= 0) {
      exit_error("inet_pton() failed - check address is valid");
    }
  }

  // Build the message string
  char msg[16];
  sprintf(msg,"ZO,%02x%02x%02x%02x\r\n",
	  0,
	  HIBYTE(id),
	  cmd,
	  LOBYTE(id));

  socklen_t sa_size = sizeof(sa);

  if (sendto(sock,&msg,strlen(msg),0,(struct sockaddr*)&sa,sizeof(sa)) <= 0) {
    std::cout << "sendto() failed\n";
  }

  close(sock);
  return 0;
}
