/* outletctl - see szInfo below for copyright and licence details */

const char szInfo[] = "\
Remote Outlet Control Client\n\
Version 1.0\n\
\n\
A simple client to send commands to the Remote Outlet Control Server.\n\
See http://www.sconemad.com for more information.\n\
\n\
Usage: outletctl <addr> id button state\n\
where:\n\
\taddr = Address of server (optional; defaults to loopback)\n\
\tid = identifier (0...65535)\n\
\tbutton = A,B,C,D or ALL\n\
\tstate = ON or OFF\n\
\n\
Copyright (c) 2009 Andrew Wedgbury <wedge@sconemad.com>\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2, or (at your option)\n\
any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program (see the file COPYING); if not, write to the\n\
Free Software Foundation, Inc.,\n\
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA\n\
";

#include <winsock2.h> 
#include <windows.h> 
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>


// The (UDP) port on which to connect
//
// 7900 is the default for the server
//
const short port = 7900;


//===========================================================================
// Popup an error message
//
void do_error(const char* msg, int flags = 0)
{
  MessageBox(0,msg,"Remote outlet control client",flags ? flags : (MB_OK | MB_ICONERROR));
  exit(1);
}


//===========================================================================
// Main program entry point
//
int APIENTRY WinMain(
  HINSTANCE /*hInstance*/,
  HINSTANCE /*hPrevInstance*/,
  LPSTR /*lpCmdLine*/,
  int /*nCmdShow*/
)
{
  // Transfer command line arguments into argv/argc style
  int argi = 0;
  TCHAR cmdline[4096] ;
  TCHAR* argv[4096] ;
  int argc = 0 ;
  _tcscpy(cmdline,GetCommandLine());
  argv[argc] = _tcstok(cmdline,TEXT(" \t"));
  while (argv[argc] != 0) {
    ++argc;
    argv[argc] = _tcstok(0,TEXT(" \t"));
  }

  // Check number of arguments
  if (argc < 4) {
    do_error(szInfo,MB_OK | MB_ICONINFORMATION);
	}

  // Server address
  char addr[64];
  if (argc == 5) {
    strcpy(addr,argv[++argi]);
  } else {
    strcpy(addr,"127.0.0.1");
  }

	// Identifier
	char* str_id = argv[++argi];
	WORD id = atoi(str_id);

	// Button
	char* str_btn = argv[++argi];
	BYTE cmd = 0;

	if (0 == stricmp(str_btn,"a")) cmd = 0xE0;
	else if (0 == stricmp(str_btn,"b")) cmd = 0xF0;
	else if (0 == stricmp(str_btn,"c")) cmd = 0x00;
	else if (0 == stricmp(str_btn,"d")) cmd = 0x10;
	else if (0 == stricmp(str_btn,"all")) cmd = 0x20;
	else {
    char msg[64];
		sprintf(msg,"Invalid button argument '%s'\n",str_btn);
    do_error(msg);
	}

  // State
	char* str_state = argv[++argi];
	if (0 == stricmp(str_state,"on")) cmd |= 0x04;
	else if (0 == stricmp(str_state,"off")) cmd |= 0x03;
	else {
    char msg[64];
		sprintf(msg,"Invalid state argument '%s'\n",str_state);
    do_error(msg);
	}

  // Init Windows sockets
  WSADATA wsaData;
  if (0 != WSAStartup(MAKEWORD(2,0),&wsaData)) {
    do_error("Unable to create socket");
  }

  // Create the socket
  SOCKET sock = WSASocket(AF_INET,SOCK_DGRAM,IPPROTO_UDP,
                     NULL,NULL,NULL);
  if (sock == INVALID_SOCKET) {
    do_error("Unable to create socket");
  }

  // Setup the address
  sockaddr_in sendAddr;
  sendAddr.sin_family = AF_INET;
  sendAddr.sin_port = htons(port);
  if (isalpha(addr[0])) {
    hostent* host = gethostbyname(addr);
    if (host == 0) {
      do_error("Unable to resolve server address");
    }
    memcpy(&sendAddr.sin_addr.S_un.S_addr,*host->h_addr_list,host->h_length);
  } else {
    sendAddr.sin_addr.s_addr = inet_addr(addr);
  }
  int sendAddrSize = sizeof(sendAddr);

  // Build the message string
	char msg[16];
	sprintf(msg,"ZO,%02x%02x%02x%02x\r\n",
		0,
		HIBYTE(id),
		cmd,
		LOBYTE(id));

  // Create data buffer for the message
  WSABUF databuf;
  databuf.len = lstrlen(msg);
  databuf.buf = msg;

  // Send the data
  DWORD na;
  if (0 != WSASendTo(sock,
                     &databuf,
                     1,
                     &na,
                     NULL,
                     (SOCKADDR*)&sendAddr,
                     sendAddrSize,
                     NULL,
                     NULL)) {

    do_error("Error sending command");
  }

  // Cleanup
  closesocket(sock);
  WSACleanup();

	return 0;
}

