/* outletsrv - see szInfo below for copyright and licence details */

const char szInfo[] = "\
Remote Outlet Control Server\n\
Version 1.0\n\
\n\
A server to control remote outlets via USB serial.\n\
See http://www.sconemad.com for more information.\n\
\n\
Copyright (c) 2009 Andrew Wedgbury <wedge@sconemad.com>\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2, or (at your option)\n\
any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program (see the file COPYING); if not, write to the\n\
Free Software Foundation, Inc.,\n\
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA\n\
";

#include <winsock2.h>
#include <windows.h>
#include <ws2tcpip.h>
#include <stdio.h>
#include "outletsrv_res.h"


// Network interface on which to listen for connections
//
// This can be either the IP address of a specific interface,
// or:
// "0.0.0.0" to allow connections from anywhere
// "127.0.0.1" to allow connections from only the local machine
//
const char* netiface = "0.0.0.0";


// The (UDP) port on which to listen
//
// I picked 7900 at random as the default. 
// Note that if you change this, you also need to change in in the client.
//
const short netport = 7900;


// Default com port
//
// This is used if one is not specified on the command line
//
const char* defcomport = "COM6";


// CONSTANTS
const char szAppName[] = "Remote Outlet Control Server";
const char szWindowClass[] = "RemoteOutletSrvCls";

// COMMAND IDS
#define IDC_ABOUT 101
#define IDC_EXIT 102

#define UM_ICON_NOTIFY WM_USER
#define UM_SOCKET_DATA WM_USER + 1

// GLOBALS
HINSTANCE g_hinstance;
char comport[16];
HWND g_hwnd;
HMENU g_hmenu;
NOTIFYICONDATA g_nid;
HICON g_hicon_normal;
HICON g_hicon_active;
HICON g_hicon_error;
HANDLE g_hcom = INVALID_HANDLE_VALUE;
SOCKET g_sock = INVALID_SOCKET;
char g_buffer[64];
WSABUF g_databuf;
WSAOVERLAPPED g_sock_ol = {0};

// PROTOTYPES
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);  


//===========================================================================
// Set the systray icon to indicate current status
//
void set_tray_icon(HICON hicon)
{
  g_nid.uFlags = NIF_ICON;
  g_nid.hIcon = hicon;
  Shell_NotifyIcon(NIM_MODIFY,&g_nid);
}

//===========================================================================
// Generate popup menu
//
void popup_menu()
{
  POINT pt;
  GetCursorPos(&pt);
  if (g_hmenu) {
    DestroyMenu(g_hmenu);
  }
  g_hmenu = CreatePopupMenu();
  AppendMenu(g_hmenu,MF_STRING,IDC_ABOUT,"&About");
  AppendMenu(g_hmenu,MF_SEPARATOR,0,0);
  AppendMenu(g_hmenu,MF_STRING,IDC_EXIT,"&Exit");
  SetForegroundWindow(g_hwnd);
  TrackPopupMenu(g_hmenu,0,pt.x,pt.y,0,g_hwnd,0);
  PostMessage(g_hwnd,WM_NULL,0,0);
}  

//===========================================================================
// Open com port
//
BOOL open_comms(const char* port)
{
  if (INVALID_HANDLE_VALUE != g_hcom) {
    CloseHandle(g_hcom);
  }

	g_hcom = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_ALWAYS, 0, 0);

  if (INVALID_HANDLE_VALUE == g_hcom) {
    return FALSE;
  }

	DCB dcb;
	FillMemory(&dcb,sizeof(DCB),0);
	if (!GetCommState(g_hcom,&dcb)) {
	  return FALSE;
	}

	dcb.DCBlength = sizeof(DCB);
	dcb.BaudRate = CBR_115200;
	dcb.fBinary = 1; 
	dcb.fParity = 1;
	dcb.fOutxCtsFlow = 0;
	dcb.fOutxDsrFlow = 0;
	dcb.fDtrControl = DTR_CONTROL_ENABLE;
	dcb.fDsrSensitivity = 0;
	dcb.fTXContinueOnXoff = 0;
	dcb.fOutX = 0;
	dcb.fInX = 0;
	dcb.fErrorChar = 0;
	dcb.fNull = 0;
	dcb.fRtsControl = RTS_CONTROL_ENABLE;
	dcb.fAbortOnError = 0;
	dcb.fDummy2 = 0;
	dcb.wReserved = 0;
	dcb.XonLim = 0x800;
	dcb.XoffLim = 0x200;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.XonChar = 0x11;
	dcb.XoffChar = 0x13;
	dcb.ErrorChar = 0;
	dcb.EofChar = 0;
	dcb.EvtChar = 0;
	dcb.wReserved1 = 0;

	if (!SetCommState(g_hcom,&dcb)) {
	  return FALSE;
	}

	return TRUE;
}

//===========================================================================
// Open socket
//
BOOL open_socket(short port)
{
  // Close the socket if it is already open
  if (g_sock != INVALID_SOCKET) {
    closesocket(g_sock);
  }

  // Create the socket
  g_sock = WSASocket(AF_INET,
                     SOCK_DGRAM,
                     IPPROTO_UDP,
                     NULL,
                     NULL,
                     NULL);  
  if (g_sock == INVALID_SOCKET) {
    return FALSE;
  }

  // Bind to address
  sockaddr_in recvAddr;
  recvAddr.sin_family = AF_INET;
  recvAddr.sin_port = htons(port);
  recvAddr.sin_addr.s_addr = inet_addr(netiface);

  if (0 != bind(g_sock,
                (SOCKADDR *)&recvAddr,
                sizeof(sockaddr_in))) {
    return FALSE;
  }

  return TRUE;
}

//===========================================================================
BOOL listen_socket()
{
  if (0 != WSAAsyncSelect(g_sock,g_hwnd,UM_SOCKET_DATA,FD_READ)) {
    return FALSE;
  }
  return TRUE;
}

//===========================================================================
BOOL on_sock_readable()
{
  g_databuf.buf = g_buffer;
  g_databuf.len = sizeof(g_buffer);
  
  DWORD na;
  DWORD flags = 0;
  sockaddr_in senderAddr;
  int senderAddrSize = sizeof(senderAddr);

  // Receive data from socket
  if (0 != WSARecvFrom(g_sock,
                       &g_databuf,
                       1,
                       &na,
                       &flags,
                       (SOCKADDR*)&senderAddr,
                       &senderAddrSize,
                       NULL,
                       NULL)) {
    return FALSE;
  }

  // Send data to serial port
	DWORD n = lstrlen(g_buffer);
	if (!WriteFile(g_hcom,g_buffer,n,&na,0)) {
    MessageBox(0,"Error sending command","Message",MB_OK);
    return FALSE;
  }

  listen_socket();
  return TRUE;
}

//===========================================================================
// Main window class
//
LRESULT CALLBACK WndProc(
  HWND hWnd,
  UINT message,
  WPARAM wParam,
  LPARAM lParam
) 
{
  switch (message) {
    case WM_CREATE: {
    } return 0;

    case WM_TIMER: {
      KillTimer(hWnd,0);
      set_tray_icon(g_hicon_normal);
    } break;

    case UM_ICON_NOTIFY: {
      switch (lParam) {
        case WM_LBUTTONDOWN:
	        break;
        case WM_RBUTTONDOWN:
          popup_menu();
	        break;
      }
    } break;

    case UM_SOCKET_DATA: {
      set_tray_icon(g_hicon_active);
      on_sock_readable();
      SetTimer(hWnd,0,500,0);
    } break;

    case WM_COMMAND: {
      switch (LOWORD(wParam)) {
		    case IDC_ABOUT:
			    MessageBox(hWnd,szInfo,szAppName,MB_OK | MB_ICONINFORMATION);
			    break;
        case IDC_EXIT:
			    PostMessage(hWnd,WM_DESTROY,0,0);
			    break;
        }
      } break;

    case WM_DESTROY: {
      KillTimer(hWnd,0);
      PostQuitMessage(0);
    } break;

    default:
      return DefWindowProc(hWnd, message, wParam, lParam);
  }
  return 0;
}

//===========================================================================
// Main program entry point
//
int APIENTRY WinMain(
  HINSTANCE hInstance,
  HINSTANCE hPrevInstance,
  LPSTR lpCmdLine,
  int nCmdShow
)
{
  g_hinstance = hInstance;

  strcpy(comport,"\\\\.\\");
  if (lpCmdLine && lstrlen(lpCmdLine) > 0) {
    strcat(comport,lpCmdLine);
  } else {
    strcat(comport,defcomport);
  }

  // Winsock startup
  WSADATA wsaData;
  if (0 != WSAStartup(MAKEWORD(2,0),&wsaData)) {
    return -1;
  }

  // Create the (hidden) main window
  WNDCLASSEX wc;
  int size = sizeof(WNDCLASSEX);
  FillMemory(&wc,size,0x00);
  wc.cbSize = size;
  wc.lpfnWndProc  = (WNDPROC)WndProc;
  wc.hInstance    = g_hinstance;
  wc.lpszClassName= szWindowClass;
  RegisterClassEx(&wc);
  g_hwnd = CreateWindow(szWindowClass,
                        szAppName,
                        WS_OVERLAPPEDWINDOW,
                        0,0,0,0,
                        NULL,
                        NULL,
                        g_hinstance,
                        NULL);
  if (!g_hwnd) {
	  return -1;
  }

  g_hicon_normal = LoadIcon(g_hinstance, MAKEINTRESOURCE(IDI_APP));
  g_hicon_active = LoadIcon(g_hinstance, MAKEINTRESOURCE(IDI_APP_BUSY));
  g_hicon_error = LoadIcon(g_hinstance, MAKEINTRESOURCE(IDI_APP_ERROR));

  // Create the systray icon
  sprintf(g_nid.szTip,"%s\nport=%s\nnet=%s:%d",szAppName,comport,netiface,netport);
  g_nid.cbSize = sizeof(NOTIFYICONDATA);
  g_nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
  g_nid.hWnd = g_hwnd;
  g_nid.hIcon = g_hicon_normal;
  g_nid.uCallbackMessage = UM_ICON_NOTIFY;
  Shell_NotifyIcon(NIM_ADD,&g_nid);

  // Open server socket
  if (!open_socket(netport)) {
    set_tray_icon(g_hicon_error);  
  }

  // Start listening for data
  if (!listen_socket()) {
    set_tray_icon(g_hicon_error);  
  }

  // Open comm port
  if (!open_comms(comport)) {
    set_tray_icon(g_hicon_error);
  }

  // Windows message loop
  MSG msg;
  while (GetMessage(&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  // Clean up
  Shell_NotifyIcon(NIM_DELETE,&g_nid);
  DestroyIcon(g_hicon_normal);
  DestroyIcon(g_hicon_active);
  DestroyIcon(g_hicon_error);
  if (g_hcom != INVALID_HANDLE_VALUE) {
    CloseHandle(g_hcom);
  }
  if (g_sock != INVALID_SOCKET) {
    closesocket(g_sock);
  }
  WSACloseEvent(g_sock_ol.hEvent);
  WSACleanup();

  // Return to system
  return msg.wParam;
}
